@@ -28,9 +28,9 @@
 ##' the delta method using \code{skip.delta.method=TRUE}.
 ##'
 ##' For random effect models a generalized delta-method is used. First
-##' the joint covariance of random effects and parameters is estimated
+##' the joint covariance of random effect and parameter estimation error is approximated
 ##' by
-##' \deqn{V \pmatrix{ \hat u \cr \hat\theta } \approx
+##' \deqn{V \pmatrix{ \hat u - u \cr \hat\theta - \theta } \approx
 ##' \pmatrix{ H_{uu}^{-1} & 0 \cr 0 & 0 } +
 ##' J V(\hat\theta) J'
 ##' }
@@ -38,19 +38,19 @@
 ##' Hessian of \code{obj$env$f} and \eqn{J} denotes the Jacobian of
 ##' \eqn{\pmatrix{\hat u(\theta) \cr \theta}} wrt. \eqn{\theta}.
 ##' Here, the first term represents the expected conditional variance
-##' given the parameters and the second term represents the variance
-##' of the conditional mean wrt. the parameters.
+##' of the estimation error given the data and the second term represents the variance
+##' of the conditional mean of the estimation error given the data.
 ##'
 ##' Now the delta method can be applied on a general non-linear
 ##' function \eqn{\phi(u,\theta)} of random effects \eqn{u} and
 ##' parameters \eqn{\theta}:
-##' \deqn{V(\phi(\hat u,\hat\theta))\approx \nabla\phi V \pmatrix{
-##' \hat u \cr \hat\theta }\nabla\phi'}
+##' \deqn{V\left(\phi(\hat u,\hat\theta) - \phi(u,\theta) \right)\approx \nabla\phi V \pmatrix{
+##' \hat u - u \cr \hat\theta - \theta }\nabla\phi'}
 ##'
 ##' The full joint covariance is not returned by default, because it
 ##' may require large amounts of memory.  It may be obtained by
 ##' specifying \code{getJointPrecision=TRUE}, in which case \eqn{V
-##' \pmatrix{ \hat u \cr \hat\theta } ^{-1} } will be part of the
+##' \pmatrix{ \hat u - u \cr \hat\theta - \theta } ^{-1} } will be part of the
 ##' output. This matrix must be manually inverted using
 ##' \code{solve(jointPrecision)} in order to get the joint covariance
 ##' matrix. Note, that the parameter order will follow the original
@@ -84,7 +84,11 @@
 ##' variables. First option is to pass a list of indices as
 ##' \code{bias.correct.control$split}. E.g. a list
 ##' \code{list(1:2,3:4)} calculates the first four ADREPORTed
-##' variables in two chunks. Second option is to pass the number of
+##' variables in two chunks.
+##' The internal function \code{obj$env$ADreportIndex()}
+##' gives an overview of the possible indices of ADREPORTed variables.
+##'
+##' Second option is to pass the number of
 ##' chunks as \code{bias.correct.control$nsplit} in which case all
 ##' ADREPORTed variables are bias corrected in the specified number of
 ##' chunks.
@@ -239,7 +243,7 @@ sdreport <- function(obj,par.fixed=NULL,hessian.fixed=NULL,getJointPrecision=FAL
           sd <- sqrt(diag(cov))
       } else {
           tmp <- lapply(seq_along(phi), doDeltaMethod)
-          sd <- sqrt(unlist(tmp))
+          sd <- sqrt(as.numeric(unlist(tmp)))
           cov <- NA
       }
   } else {
@@ -329,9 +333,7 @@ sdreport <- function(obj,par.fixed=NULL,hessian.fixed=NULL,getJointPrecision=FAL
   ## ======== Find marginal variances of all random effects i.e. phi(u,theta)=u
   if(!is.null(r)){
     if(is(L,"dCHMsuper")){ ## Required by inverse subset algorithm
-      ihessian.random <- .Call("tmb_invQ", L, PACKAGE = "TMB")
-      iperm <- invPerm(L@perm+1L)
-      diag.term1 <- diag(ihessian.random)[iperm]
+      diag.term1 <- solveSubset(L=L, diag=TRUE)
       if(ignore.parm.uncertainty){
           diag.term2 <- 0
       } else {
@@ -381,6 +383,7 @@ sdreport <- function(obj,par.fixed=NULL,hessian.fixed=NULL,getJointPrecision=FAL
   ans$env <- new.env(parent = emptyenv())
   ans$env$parameters <- obj$env$parameters
   ans$env$random <- obj$env$random
+  ans$env$ADreportDims <- obj2$env$ADreportDims
   class(ans) <- "sdreport"
   ans
 }
@@ -461,7 +464,8 @@ print.sdreport <- function(x, ...)
 ##'
 ##' @title Convert estimates to original list format.
 ##' @param x Output from \code{\link{sdreport}}.
-##' @param what Select what to convert.
+##' @param what Select what to convert (Estimate / Std. Error).
+##' @param report Get AD reported variables rather than model parameters ?
 ##' @param ... Passed to \code{\link{summary.sdreport}}.
 ##' @return List of same shape as original parameter list.
 ##' @method as.list sdreport
@@ -469,44 +473,74 @@ print.sdreport <- function(x, ...)
 ##' @examples
 ##' \dontrun{
 ##' example(sdreport)
+##'
+##' ## Estimates as a parameter list:
 ##' as.list(rep, "Est")
+##'
+##' ## Std Errors in the same list format:
 ##' as.list(rep, "Std")
+##'
+##' ## p-values in the same list format:
 ##' as.list(rep, "Pr", p.value=TRUE)
+##'
+##' ## AD reported variables as a list:
+##' as.list(rep, "Estimate", report=TRUE)
+##'
+##' ## Bias corrected AD reported variables as a list:
+##' as.list(rep, "Est. (bias.correct)", report=TRUE)
 ##' }
-as.list.sdreport <- function(x, what = "", ...){
-    ans <- x$env$parameters
-    random <- x$env$random
-    par <- numeric(length(x$par.fixed) +
-                   length(x$par.random))
-    fixed <- rep(TRUE, length(par))
-    if(length(random)>0)
-        fixed[random] <- FALSE
-    ## Possible choices
-    opts <- colnames( summary(x, select = c("fixed", "random"), ...) )
-    what <- match.arg(what, opts)
-    if( any( fixed ) )
-        par[ fixed ] <- summary(x, select = "fixed",  ...)[ , what]
-    if( any(!fixed ) )
-        par[!fixed ] <- summary(x, select = "random", ...)[ , what]
-    ## Workaround utils::relist bug (?) for empty list items
-    nonemp <- sapply(ans, function(x)length(x) > 0)
-    nonempindex <- which(nonemp)
-    skeleton <- as.relistable(ans[nonemp])
-    li <- relist(par, skeleton)
-    reshape <- function(x){
-        if(is.null(attr(x,"map")))
-            return(x)
-        y <- attr(x,"shape")
-        f <- attr(x,"map")
-        i <- which(f >= 0)
-        y[i] <- x[f[i] + 1L]
-        y
-    }
-    for(i in seq(skeleton)){
-        ans[[nonempindex[i]]][] <- as.vector(li[[i]])
-    }
-    for(i in seq(ans)){
-        ans[[i]] <- reshape(ans[[i]])
+as.list.sdreport <- function(x, what = "", report=FALSE, ...) {
+    if (!report) {
+        ans <- x$env$parameters
+        random <- x$env$random
+        par <- numeric(length(x$par.fixed) +
+                       length(x$par.random))
+        fixed <- rep(TRUE, length(par))
+        if(length(random)>0)
+            fixed[random] <- FALSE
+        ## Possible choices
+        opts <- colnames( summary(x, select = c("fixed", "random"), ...) )
+        what <- match.arg(what, opts)
+        if( any( fixed ) )
+            par[ fixed ] <- summary(x, select = "fixed",  ...)[ , what]
+        if( any(!fixed ) )
+            par[!fixed ] <- summary(x, select = "random", ...)[ , what]
+        ## Workaround utils::relist bug (?) for empty list items
+        nonemp <- sapply(ans, function(x)length(x) > 0)
+        nonempindex <- which(nonemp)
+        skeleton <- as.relistable(ans[nonemp])
+        li <- relist(par, skeleton)
+        reshape <- function(x){
+            if(is.null(attr(x,"map")))
+                return(x)
+            y <- attr(x,"shape")
+            ## Handle special case where parameters are mapped to a fixed
+            ## value
+            if (what != "Estimate") {
+                y[] <- NA
+            }
+            f <- attr(x,"map")
+            i <- which(f >= 0)
+            y[i] <- x[f[i] + 1L]
+            y
+        }
+        for(i in seq(skeleton)){
+            ans[[nonempindex[i]]][] <- as.vector(li[[i]])
+        }
+        for(i in seq(ans)){
+            ans[[i]] <- reshape(ans[[i]])
+        }
+    } else { ## Reported variables
+        ## Possible choices
+        opts <- colnames( summary(x, select = "report", ...) )
+        what <- match.arg(what, opts)
+        par <- summary(x, select = "report",  ...)[ , what]
+        skeleton <- lapply(x$env$ADreportDims,
+                           function(dim) array(NA, dim))
+        skeleton <- as.relistable(skeleton)
+        ans <- relist(par, skeleton) ## Not keeping array dims !
+        ans <- Map(array, ans, x$env$ADreportDims)
+        class(ans) <- NULL
     }
     attr(ans, "check.passed") <- NULL
     attr(ans, "what") <- what
